@@ -23,6 +23,12 @@
 /* FIXME: refactoring */
 /* FIXME: setcmpfunc tcbdbcmplexical/decimal/cmpint32/cmpint64' */
 
+typedef enum {
+  iter_key = 0,
+  iter_value,
+  iter_item
+} itertype;
+
 /* Error Objects */
 
 static PyObject *PyTCError;
@@ -55,6 +61,8 @@ raise_tcbdb_error(TCBDB *bdb) {
 typedef struct {
   PyObject_HEAD
   TCHDB	*hdb;
+  itertype itype;
+  bool hold_itype;
 } PyTCHDB;
 
 typedef struct {
@@ -68,6 +76,7 @@ typedef struct {
   PyObject_HEAD
   PyTCBDB *bdb;
   BDBCUR *cur;
+  itertype itype;
 } PyBDBCUR;
 
 /* Macros */
@@ -297,6 +306,176 @@ typedef struct {
     return ret; \
   }
 
+/* for dict like interface */
+#define TCXDB_length(func,type,call,member) \
+  static long \
+  func(type *self) { \
+    return call(self->member); \
+  } \
+
+#define TCXDB_subscript(func,type,call,member,err) \
+  static PyObject * \
+  func(type *self, PyObject *_key) { \
+    PyObject *ret; \
+    char *key, *value; \
+    int key_len, value_len; \
+  \
+    if (!PyString_Check(_key)) { \
+      PyErr_SetString(PyExc_TypeError, "only string is allowed in []"); \
+      return NULL; \
+    } \
+    key = PyString_AS_STRING(_key); \
+    key_len = PyString_GET_SIZE(_key); \
+    if (!key || !key_len) { \
+      return NULL; \
+    } \
+    Py_BEGIN_ALLOW_THREADS \
+    value = call(self->member, key, key_len, &value_len); \
+    Py_END_ALLOW_THREADS \
+  \
+    if (!value) { \
+      err(self->member); \
+      return NULL; \
+    } \
+    ret = PyString_FromStringAndSize(value, value_len); \
+    free(value); \
+    return ret; \
+  } \
+
+#define TCXDB_DelItem(func,type,call,member,err) \
+  int \
+  func(type *self, PyObject *_key) { \
+    bool result; \
+    char *key = PyString_AS_STRING(_key); \
+    int key_len = PyString_GET_SIZE(_key); \
+  \
+    if (!key || !key_len) { \
+      return -1; \
+    } \
+    Py_BEGIN_ALLOW_THREADS \
+    result = call(self->member, key, key_len); \
+    Py_END_ALLOW_THREADS \
+  \
+    if (!result) { \
+      err(self->member); \
+      return -1; \
+    } \
+    return 0; \
+  }
+
+#define TCXDB_SetItem(func,type,call,member,err) \
+  int \
+  func(type *self, PyObject *_key, PyObject *_value) { \
+    bool result; \
+    char *key = PyString_AS_STRING(_key), *value = PyString_AS_STRING(_value); \
+    int key_len = PyString_GET_SIZE(_key), value_len = PyString_GET_SIZE(_value); \
+  \
+    if (!key || !key_len || !value) { \
+      return -1; \
+    } \
+    Py_BEGIN_ALLOW_THREADS \
+    result = call(self->member, key, key_len, value, value_len); \
+    Py_END_ALLOW_THREADS \
+  \
+    if (!result) { \
+      err(self->member); \
+      return -1; \
+    } \
+    return 0; \
+  }
+
+#define TCXDB_ass_sub(func,type,setitem,delitem) \
+  static int \
+  func(type *self, PyObject *v, PyObject *w) \
+  { \
+    if (w) { \
+      return setitem(self, v, w); \
+    } else { \
+      return delitem(self, v); \
+    } \
+  }
+
+#define TCXDB_Contains(func,type,call,member) \
+  static int \
+  func(type *self, PyObject *_key) { \
+    char *key = PyString_AS_STRING(_key); \
+    int key_len = PyString_GET_SIZE(_key), value_len; \
+  \
+    if (!key || !key_len) { \
+      return -1; \
+    } \
+    Py_BEGIN_ALLOW_THREADS \
+    value_len = call(self->member, key, key_len); \
+    Py_END_ALLOW_THREADS \
+  \
+    return (value_len != -1); \
+  }
+
+#define TCXDB___contains__(func,type,call) \
+  static PyObject * \
+  func(type *self, PyObject *_key) { \
+    return PyBool_FromLong(call(self, _key) != 0); \
+  }
+
+#define TCXDB___getitem__(func,type,call,member,err) \
+  static PyObject * \
+  func(type *self, PyObject *_key) { \
+    PyObject *ret; \
+    char *key = PyString_AS_STRING(_key), *value; \
+    int key_len = PyString_GET_SIZE(_key), value_len; \
+  \
+    if (!key || !key_len) { \
+      return NULL; \
+    } \
+    Py_BEGIN_ALLOW_THREADS \
+    value = call(self->member, key, key_len, &value_len); \
+    Py_END_ALLOW_THREADS \
+  \
+    if (!value) { \
+      err(self->member); \
+      return NULL; \
+    } \
+    ret = PyString_FromStringAndSize(value, value_len); \
+    free(value); \
+    return ret; \
+  }
+
+#define TCXDB_iters(type,funcikey,funcival,funciitem,call) \
+  static PyObject * \
+  funcikey(type *self) { \
+    return call(self, iter_key); \
+  } \
+  \
+  static PyObject * \
+  funcival(type *self) { \
+    return call(self, iter_value); \
+  } \
+  \
+  static PyObject * \
+  funciitem(type *self) { \
+    return call(self, iter_item); \
+  }
+
+/* for TCXSTR key/value */
+#define GET_TCXSTR_KEY_VALUE(call,callarg) \
+  PyObject *ret = NULL; \
+  TCXSTR *key, *value; \
+  \
+  key = tcxstrnew(); \
+  value = tcxstrnew(); \
+  if (key && value) { \
+    bool result; \
+  \
+    Py_BEGIN_ALLOW_THREADS \
+    result = call(callarg, key, value); \
+    Py_END_ALLOW_THREADS
+
+#define CLEAR_TCXSTR_KEY_VALUE() \
+  } \
+  if (key) { tcxstrdel(key); } \
+  if (value) { tcxstrdel(value); } \
+  return ret;
+
 /*** TCHDB ***/
 
 #define PyTCHDB_TUNE(a,b,c) \
@@ -414,30 +593,54 @@ INT_KEYARGS(PyTCHDB_vsiz,PyTCHDB,vsiz,tchdbvsiz,hdb,raise_tchdb_error)
 BOOL_NOARGS(PyTCHDB_iterinit,PyTCHDB,tchdbiterinit,hdb,raise_tchdb_error,hdb)
 
 static PyObject *
-PyTCHDB_GetIter(PyObject *self) {
-  if (PyTCHDB_iterinit((PyTCHDB *)self)) {
+PyTCHDB_GetIter(PyTCHDB *self, itertype itype) {
+  if (PyTCHDB_iterinit(self)) {
     Py_INCREF(self);
-    return self;
+    /* hack */
+    if (self->hold_itype) {
+      self->hold_itype = false;
+    } else {
+      self->itype = itype;
+      if (itype != iter_key) {
+        self->hold_itype = true;
+      }
+    }
+    return (PyObject *)self;
   }
   return NULL;
 }
 
+TCXDB_iters(PyTCHDB,PyTCHDB_GetIter_keys,PyTCHDB_GetIter_values,PyTCHDB_GetIter_items,PyTCHDB_GetIter)
+
 static PyObject *
 PyTCHDB_iternext(PyTCHDB *self) {
-  void *key;
-  int key_len;
+  if (self->itype == iter_key) {
+    void *key;
+    int key_len;
 
-  Py_BEGIN_ALLOW_THREADS
-  key = tchdbiternext(self->hdb, &key_len);
-  Py_END_ALLOW_THREADS
+    Py_BEGIN_ALLOW_THREADS
+    key = tchdbiternext(self->hdb, &key_len);
+    Py_END_ALLOW_THREADS
 
-  if (key) {
-    PyObject *_key;
-    _key = PyString_FromStringAndSize(key, key_len);
-    free(key);
-    return _key;
+    if (key) {
+      PyObject *_key;
+      _key = PyString_FromStringAndSize(key, key_len);
+      free(key);
+      return _key;
+    }
+    return NULL;
+  } else {
+    GET_TCXSTR_KEY_VALUE(tchdbiternext3,self->hdb)
+    if (result) {
+      if (self->itype == iter_value) {
+        ret = PyString_FromStringAndSize(tcxstrptr(value), tcxstrsize(value));
+      } else {
+        ret = Py_BuildValue("(s#s#)", tcxstrptr(key), tcxstrsize(key),
+                                      tcxstrptr(value), tcxstrsize(value));
+      }
+    }
+    CLEAR_TCXSTR_KEY_VALUE()
   }
-  return NULL;
 }
 
 BOOL_NOARGS(PyTCHDB_sync,PyTCHDB,tchdbsync,hdb,raise_tchdb_error,hdb)
@@ -450,48 +653,9 @@ BOOL_PATHARGS(PyTCHDB_copy, PyTCHDB, copy, tchdbcopy, hdb, raise_tchdb_error)
 
 /* TODO: features for experts */
 
-static int
-PyTCHDB_Contains(PyTCHDB *self, PyObject *_key) {
-  char *key = PyString_AS_STRING(_key);
-  int key_len = PyString_GET_SIZE(_key), value_len;
-
-  if (!key || !key_len) {
-    return -1;
-  }
-  Py_BEGIN_ALLOW_THREADS
-  value_len = tchdbvsiz(self->hdb, key, key_len);
-  Py_END_ALLOW_THREADS
-
-  return (value_len != -1);
-}
-
-static PyObject *
-PyTCHDB___contains__(PyTCHDB *self, PyObject *_key) {
-  return PyBool_FromLong(PyTCHDB_Contains(self, _key) != 0);
-}
-
-static PyObject *
-PyTCHDB___getitem__(PyTCHDB *self, PyObject *_key) {
-  PyObject *ret;
-  char *key = PyString_AS_STRING(_key), *value;
-  int key_len = PyString_GET_SIZE(_key), value_len;
-
-  if (!key || !key_len) {
-    return NULL;
-  }
-  Py_BEGIN_ALLOW_THREADS
-  value = tchdbget(self->hdb, key, key_len, &value_len);
-  Py_END_ALLOW_THREADS
-
-  if (!value) {
-    raise_tchdb_error(self->hdb);
-    return NULL;
-  }
-  ret = PyString_FromStringAndSize(value, value_len);
-  free(value);
-  return ret;
-}
-
+TCXDB_Contains(PyTCHDB_Contains,PyTCHDB,tchdbvsiz,hdb)
+TCXDB___contains__(PyTCHDB___contains__,PyTCHDB,PyTCHDB_Contains)
+TCXDB___getitem__(PyTCHDB___getitem__,PyTCHDB,tchdbget,hdb,raise_tchdb_error)
 TCXDB_rnum(TCHDB_rnum,TCHDB,tchdbrnum)
 
 static PyObject *
@@ -612,82 +776,11 @@ PyTCHDB_values(PyTCHDB *self) {
   return ret;
 }
 
-static long
-PyTCHDB_length(PyTCHDB *self) {
-  return TCHDB_rnum(self->hdb);
-}
-
-static PyObject *
-PyTCHDB_subscript(PyTCHDB *self, PyObject *_key) {
-  PyObject *ret;
-  char *key = PyString_AS_STRING(_key), *value;
-  int key_len = PyString_GET_SIZE(_key), value_len;
-
-  if (!key || !key_len) {
-    return NULL;
-  }
-  Py_BEGIN_ALLOW_THREADS
-  value = tchdbget(self->hdb, key, key_len, &value_len);
-  Py_END_ALLOW_THREADS
-
-  if (!value) {
-    raise_tchdb_error(self->hdb);
-    return NULL;
-  }
-  ret = PyString_FromStringAndSize(value, value_len);
-  free(value);
-  return ret;
-}
-
-int
-PyTCHDB_DelItem(PyTCHDB *self, PyObject *_key) {
-  bool result;
-  char *key = PyString_AS_STRING(_key);
-  int key_len = PyString_GET_SIZE(_key);
-
-  if (!key || !key_len) {
-    return -1;
-  }
-  Py_BEGIN_ALLOW_THREADS
-  result = tchdbout(self->hdb, key, key_len);
-  Py_END_ALLOW_THREADS
-
-  if (!result) {
-    raise_tchdb_error(self->hdb);
-    return -1;
-  }
-  return 0;
-}
-
-int
-PyTCHDB_SetItem(PyTCHDB *self, PyObject *_key, PyObject *_value) {
-  bool result;
-  char *key = PyString_AS_STRING(_key), *value = PyString_AS_STRING(_value);
-  int key_len = PyString_GET_SIZE(_key), value_len = PyString_GET_SIZE(_value);
-
-  if (!key || !key_len || !value) {
-    return -1;
-  }
-  Py_BEGIN_ALLOW_THREADS
-  result = tchdbput(self->hdb, key, key_len, value, value_len);
-  Py_END_ALLOW_THREADS
-
-  if (!result) {
-    raise_tchdb_error(self->hdb);
-    return -1;
-  }
-  return 0;
-}
-
-static int
-PyTCHDB_ass_sub(PyTCHDB *self, PyObject *v, PyObject *w)
-{
-  if (w) {
-    return PyTCHDB_SetItem(self, v, w);
-  } else {
-    return PyTCHDB_DelItem(self, v);
-  }
-}
+TCXDB_length(PyTCHDB_length,PyTCHDB,TCHDB_rnum,hdb)
+TCXDB_subscript(PyTCHDB_subscript,PyTCHDB,tchdbget,hdb,raise_tchdb_error)
+TCXDB_DelItem(PyTCHDB_DelItem,PyTCHDB,tchdbout,hdb,raise_tchdb_error)
+TCXDB_SetItem(PyTCHDB_SetItem,PyTCHDB,tchdbput,hdb,raise_tchdb_error)
+TCXDB_ass_sub(PyTCHDB_ass_sub,PyTCHDB,PyTCHDB_SetItem,PyTCHDB_DelItem)
 
 /* methods of classes */
 static PyMethodDef PyTCHDB_methods[] = {
@@ -775,6 +868,15 @@ static PyMethodDef PyTCHDB_methods[] = {
   {"values", (PyCFunction)PyTCHDB_values,
    METH_NOARGS,
    ""},
+  {"iteritems", (PyCFunction)PyTCHDB_GetIter_items,
+   METH_NOARGS,
+   ""},
+  {"iterkeys", (PyCFunction)PyTCHDB_GetIter_keys,
+   METH_NOARGS,
+   ""},
+  {"itervalues", (PyCFunction)PyTCHDB_GetIter_values,
+   METH_NOARGS,
+   ""},
   /*
   {"", (PyCFunction)PyTCHDB_,
    METH_VARARGS | METH_KEYWORDS,
@@ -831,7 +933,7 @@ static PyTypeObject PyTCHDB_Type = {
   0,                                           /* tp_clear */
   0,                                           /* tp_richcompare */
   0,                                           /* tp_weaklistoffset */
-  PyTCHDB_GetIter,                             /* tp_iter */
+  (getiterfunc)PyTCHDB_GetIter_keys,           /* tp_iter */
   (iternextfunc)PyTCHDB_iternext,              /* tp_iternext */
   PyTCHDB_methods,                             /* tp_methods */
   0,                                           /* tp_members */
@@ -925,50 +1027,39 @@ STRINGL_NOARGS(PyBDBCUR_val,PyBDBCUR,tcbdbcurval,cur,raise_tcbdb_error,bdb->bdb)
 
 static PyObject *
 PyBDBCUR_rec(PyBDBCUR *self) {
-  PyObject *ret = NULL;
-  TCXSTR *key, *value;
-
-  key = tcxstrnew();
-  value = tcxstrnew();
-  if (key && value) {
-    bool result;
-
-    Py_BEGIN_ALLOW_THREADS
-    result = tcbdbcurrec(self->cur, key, value);
-    Py_END_ALLOW_THREADS
-
-    if (result) {
-      ret = Py_BuildValue("(s#s#)", tcxstrptr(key), tcxstrsize(key),
-                                    tcxstrptr(value), tcxstrsize(value));
-    }
+  GET_TCXSTR_KEY_VALUE(tcbdbcurrec,self->cur)
+  if (result) {
+    ret = Py_BuildValue("(s#s#)", tcxstrptr(key), tcxstrsize(key),
+                                  tcxstrptr(value), tcxstrsize(value));
   }
   if (!ret) {
     raise_tcbdb_error(self->bdb->bdb);
   }
-  if (key) { tcxstrdel(key); }
-  if (value) { tcxstrdel(value); }
-  return ret;
+  CLEAR_TCXSTR_KEY_VALUE()
 }
 
 static PyObject *
 PyBDBCUR_iternext(PyBDBCUR *self) {
-  char *key;
-  int key_len;
-  PyObject *ret;
-
-  Py_BEGIN_ALLOW_THREADS
-  key = tcbdbcurkey(self->cur, &key_len);
-  Py_END_ALLOW_THREADS
-
-  if (key) {
-    ret = PyString_FromStringAndSize(key, key_len);
-    free(key);
-    Py_BEGIN_ALLOW_THREADS
-    tcbdbcurnext(self->cur);
-    Py_END_ALLOW_THREADS
-    return ret;
+  /* TODO: performance up */
+  GET_TCXSTR_KEY_VALUE(tcbdbcurrec,self->cur)
+  if (result) {
+    switch (self->itype) {
+      case iter_key:
+        ret = PyString_FromStringAndSize(tcxstrptr(key), tcxstrsize(key));
+        break;
+      case iter_value:
+        ret = PyString_FromStringAndSize(tcxstrptr(value), tcxstrsize(value));
+        break;
+      case iter_item:
+        ret = Py_BuildValue("(s#s#)", tcxstrptr(key), tcxstrsize(key),
+                                      tcxstrptr(value), tcxstrsize(value));
+        break;
+    }
   }
-  return NULL;
+  Py_BEGIN_ALLOW_THREADS
+  result = tcbdbcurnext(self->cur);
+  Py_END_ALLOW_THREADS
+  CLEAR_TCXSTR_KEY_VALUE()
 }
 
 static PyMethodDef PyBDBCUR_methods[] = {
@@ -1165,20 +1256,25 @@ BOOL_NOARGS(PyTCBDB_setmutex,PyTCBDB,tcbdbsetmutex,bdb,raise_tcbdb_error,bdb)
 static int
 TCBDB_cmpfunc(const char *aptr, int asiz,
               const char *bptr, int bsiz, PyTCBDB *self) {
-  int ret;
+  int ret = 0;
   PyObject *args, *result;
+  PyGILState_STATE gstate;
+
   if ((args = Py_BuildValue("(s#s#O)", aptr, asiz, bptr, bsiz, self->cmpop))) {
     /* TODO: set error */
     return 0;
   }
+  gstate = PyGILState_Ensure();
   result = PyEval_CallObject(self->cmp, args);
   Py_DECREF(args);
   if (!result) {
     /* TODO: set error */
-    return 0;
+    goto exit;
   }
   ret = PyLong_AsLong(result);
   Py_DECREF(result);
+exit:
+  PyGILState_Release(gstate);
   return ret;
 }
 
@@ -1339,47 +1435,9 @@ PY_U_LONG_LONG_NOARGS(PyTCBDB_fsiz, PyTCBDB, tcbdbrnum, bdb, tcbdbecode, raise_t
 
 /* TODO: features for experts */
 
-static int
-PyTCBDB_Contains(PyTCBDB *self, PyObject *_key) {
-  char *key = PyString_AS_STRING(_key);
-  int key_len = PyString_GET_SIZE(_key), value_len;
-
-  if (!key || !key_len) {
-    return -1;
-  }
-  Py_BEGIN_ALLOW_THREADS
-  value_len = tcbdbvsiz(self->bdb, key, key_len);
-  Py_END_ALLOW_THREADS
-
-  return (value_len != -1);
-}
-
-static PyObject *
-PyTCBDB___contains__(PyTCBDB *self, PyObject *_key) {
-  return PyBool_FromLong(PyTCBDB_Contains(self, _key) != 0);
-}
-
-static PyObject *
-PyTCBDB___getitem__(PyTCBDB *self, PyObject *_key) {
-  PyObject *ret;
-  char *key = PyString_AS_STRING(_key), *value;
-  int key_len = PyString_GET_SIZE(_key), value_len;
-
-  if (!key || !key_len) {
-    return NULL;
-  }
-  Py_BEGIN_ALLOW_THREADS
-  value = tcbdbget(self->bdb, key, key_len, &value_len);
-  Py_END_ALLOW_THREADS
-
-  if (!value) {
-    raise_tcbdb_error(self->bdb);
-    return NULL;
-  }
-  ret = PyString_FromStringAndSize(value, value_len);
-  free(value);
-  return ret;
-}
+TCXDB_Contains(PyTCBDB_Contains,PyTCBDB,tcbdbvsiz,bdb)
+TCXDB___contains__(PyTCBDB___contains__,PyTCBDB,PyTCBDB_Contains)
+TCXDB___getitem__(PyTCBDB___getitem__,PyTCBDB,tcbdbget,bdb,raise_tcbdb_error)
 
 static PyObject *
 PyTCBDB_curnew(PyTCBDB *self) {
@@ -1398,9 +1456,10 @@ PyTCBDB_curnew(PyTCBDB *self) {
 }
 
 static PyObject *
-PyTCBDB_GetIter(PyObject *self) {
+PyTCBDB_GetIter(PyTCBDB *self, itertype itype) {
   PyBDBCUR *cur;
-  if ((cur = (PyBDBCUR *)PyTCBDB_curnew((PyTCBDB *)self))) {
+  if ((cur = (PyBDBCUR *)PyTCBDB_curnew(self))) {
+    cur->itype = itype;
     if (PyBDBCUR_first(cur)) {
       return (PyObject *)cur;
     }
@@ -1409,6 +1468,8 @@ PyTCBDB_GetIter(PyObject *self) {
   return NULL;
 }
 
+TCXDB_iters(PyTCBDB,PyTCBDB_GetIter_keys,PyTCBDB_GetIter_values,PyTCBDB_GetIter_items,PyTCBDB_GetIter)
+
 TCXDB_rnum(TCBDB_rnum,TCBDB,tcbdbrnum)
 
 static PyObject *
@@ -1566,82 +1627,11 @@ PyTCBDB_values(PyTCBDB *self) {
   return ret;
 }
 
-static long
-PyTCBDB_length(PyTCBDB *self) {
-  return TCBDB_rnum(self->bdb);
-}
-
-static PyObject *
-PyTCBDB_subscript(PyTCBDB *self, PyObject *_key) {
-  PyObject *ret;
-  char *key = PyString_AS_STRING(_key), *value;
-  int key_len = PyString_GET_SIZE(_key), value_len;
-
-  if (!key || !key_len) {
-    return NULL;
-  }
-  Py_BEGIN_ALLOW_THREADS
-  value = tcbdbget(self->bdb, key, key_len, &value_len);
-  Py_END_ALLOW_THREADS
-
-  if (!value) {
-    raise_tcbdb_error(self->bdb);
-    return NULL;
-  }
-  ret = PyString_FromStringAndSize(value, value_len);
-  free(value);
-  return ret;
-}
-
-int
-PyTCBDB_DelItem(PyTCBDB *self, PyObject *_key) {
-  bool result;
-  char *key = PyString_AS_STRING(_key);
-  int key_len = PyString_GET_SIZE(_key);
-
-  if (!key || !key_len) {
-    return -1;
-  }
-  Py_BEGIN_ALLOW_THREADS
-  result = tcbdbout(self->bdb, key, key_len);
-  Py_END_ALLOW_THREADS
-
-  if (!result) {
-    raise_tcbdb_error(self->bdb);
-    return -1;
-  }
-  return 0;
-}
-
-int
-PyTCBDB_SetItem(PyTCBDB *self, PyObject *_key, PyObject *_value) {
-  bool result;
-  char *key = PyString_AS_STRING(_key), *value = PyString_AS_STRING(_value);
-  int key_len = PyString_GET_SIZE(_key), value_len = PyString_GET_SIZE(_value);
-
-  if (!key || !key_len || !value) {
-    return -1;
-  }
-  Py_BEGIN_ALLOW_THREADS
-  result = tcbdbput(self->bdb, key, key_len, value, value_len);
-  Py_END_ALLOW_THREADS
-
-  if (!result) {
-    raise_tcbdb_error(self->bdb);
-    return -1;
-  }
-  return 0;
-}
-
-static int
-PyTCBDB_ass_sub(PyTCBDB *self, PyObject *v, PyObject *w)
-{
-  if (w) {
-    return PyTCBDB_SetItem(self, v, w);
-  } else {
-    return PyTCBDB_DelItem(self, v);
-  }
-}
+TCXDB_length(PyTCBDB_length,PyTCBDB,TCBDB_rnum,bdb)
+TCXDB_subscript(PyTCBDB_subscript,PyTCBDB,tcbdbget,bdb,raise_tcbdb_error)
+TCXDB_DelItem(PyTCBDB_DelItem,PyTCBDB,tcbdbout,bdb,raise_tcbdb_error)
+TCXDB_SetItem(PyTCBDB_SetItem,PyTCBDB,tcbdbput,bdb,raise_tcbdb_error)
+TCXDB_ass_sub(PyTCBDB_ass_sub,PyTCBDB,PyTCBDB_SetItem,PyTCBDB_DelItem)
 
 static PyMethodDef PyTCBDB_methods[] = {
   {"errmsg", (PyCFunction)PyTCBDB_errmsg,
@@ -1753,6 +1743,15 @@ static PyMethodDef PyTCBDB_methods[] = {
   {"values", (PyCFunction)PyTCBDB_values,
    METH_NOARGS,
    ""},
+  {"iteritems", (PyCFunction)PyTCBDB_GetIter_items,
+   METH_NOARGS,
+   ""},
+  {"iterkeys", (PyCFunction)PyTCBDB_GetIter_keys,
+   METH_NOARGS,
+   ""},
+  {"itervalues", (PyCFunction)PyTCBDB_GetIter_values,
+   METH_NOARGS,
+   ""},
   /*
   {"", (PyCFunction)PyTCBDB_,
    METH_VARARGS | METH_KEYWORDS,
@@ -1810,7 +1809,7 @@ static PyTypeObject PyTCBDB_Type = {
   0,                                           /* tp_clear */
   0,                                           /* tp_richcompare */
   0,                                           /* tp_weaklistoffset */
-  PyTCBDB_GetIter,                             /* tp_iter */
+  (getiterfunc)PyTCBDB_GetIter_keys,           /* tp_iter */
   0,                                           /* tp_iternext */
   PyTCBDB_methods,                             /* tp_methods */
   0,                                           /* tp_members */
